/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_OBJECTINTENTION_H
#define DPC_IMPL_TYPE_OBJECTINTENTION_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "impl_type_int8.h"
#include "dpc/impl_type_trajectorypoint.h"

namespace dpc {
struct ObjectIntention {
    ::UInt8 intention_type{ 0U };
    ::Float intention_propability{ 0.0F };
    ::Float time_to_collision{ 0.0F };
    ::Float collision_probability{ 0.0F };
    ::UInt8 target_lane{ 0U };
    ::Float time_to_lane{ 0.0F };
    ::Int8 number_of_trajectory_points{ 0 };
    ::dpc::TrajectoryPoint trajectory_point_list;

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(intention_type);
        fun(intention_propability);
        fun(time_to_collision);
        fun(collision_probability);
        fun(target_lane);
        fun(time_to_lane);
        fun(number_of_trajectory_points);
        fun(trajectory_point_list);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(intention_type);
        fun(intention_propability);
        fun(time_to_collision);
        fun(collision_probability);
        fun(target_lane);
        fun(time_to_lane);
        fun(number_of_trajectory_points);
        fun(trajectory_point_list);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("intention_type", intention_type);
        fun("intention_propability", intention_propability);
        fun("time_to_collision", time_to_collision);
        fun("collision_probability", collision_probability);
        fun("target_lane", target_lane);
        fun("time_to_lane", time_to_lane);
        fun("number_of_trajectory_points", number_of_trajectory_points);
        fun("trajectory_point_list", trajectory_point_list);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("intention_type", intention_type);
        fun("intention_propability", intention_propability);
        fun("time_to_collision", time_to_collision);
        fun("collision_probability", collision_probability);
        fun("target_lane", target_lane);
        fun("time_to_lane", time_to_lane);
        fun("number_of_trajectory_points", number_of_trajectory_points);
        fun("trajectory_point_list", trajectory_point_list);
    }

    friend bool operator==(const ::dpc::ObjectIntention& lhs, const ::dpc::ObjectIntention& rhs) noexcept
    {
        return (lhs.intention_type == rhs.intention_type) && (fabs(static_cast<double>(lhs.intention_propability - rhs.intention_propability)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.time_to_collision - rhs.time_to_collision)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.collision_probability - rhs.collision_probability)) < DBL_EPSILON) && (lhs.target_lane == rhs.target_lane) && (fabs(static_cast<double>(lhs.time_to_lane - rhs.time_to_lane)) < DBL_EPSILON) && (lhs.number_of_trajectory_points == rhs.number_of_trajectory_points) && (lhs.trajectory_point_list == rhs.trajectory_point_list);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_OBJECTINTENTION_H
